import React, { useState, useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { setUserGoal, getConversationSummary, getRecommendedReply } from '../../store/talentChat/talentChat.thunks';
import { showNotification } from '../../store/notification.slice';
import SafeMarkdown from '../../components/SafeMarkdown';

export default function ChatSidePanel({ chatId, talentEmail }) {
    const dispatch = useDispatch();
    const [goal, setGoal] = useState('');
    const [loading, setLoading] = useState({
        goal: false,
        summary: false,
        reply: false,
    });
    const [copySuccess, setCopySuccess] = useState(false);

    // Get data from Redux store
    const userGoal = useSelector((state) => state.talentChat?.userGoal?.data);
    const conversationSummary = useSelector((state) => state.talentChat?.conversationSummary?.data);
    const recommendedReply = useSelector((state) => state.talentChat?.recommendedReply?.data);

    // Load conversation summary and recommended reply when component mounts
    useEffect(() => {
        if (talentEmail && chatId) {
            handleGetSummary();
            handleGetRecommendedReply();
        }
    }, [chatId, talentEmail]);

    // Reset copy success state after 2 seconds
    useEffect(() => {
        if (copySuccess) {
            const timer = setTimeout(() => {
                setCopySuccess(false);
            }, 2000);
            return () => clearTimeout(timer);
        }
    }, [copySuccess]);

    const handleGoalSubmit = async (e) => {
        e.preventDefault();
        if (!goal.trim()) return;

        setLoading((prev) => ({ ...prev, goal: true }));
        try {
            await dispatch(setUserGoal(goal)).unwrap();
            dispatch(
                showNotification({
                    message: '聊天目标设置成功',
                    type: 'success',
                })
            );
        } catch (error) {
            dispatch(
                showNotification({
                    message: `设置目标失败: ${error}`,
                    type: 'danger',
                })
            );
        } finally {
            setLoading((prev) => ({ ...prev, goal: false }));
        }
    };

    const handleGetSummary = async () => {
        if (!talentEmail) return;

        setLoading((prev) => ({ ...prev, summary: true }));
        try {
            await dispatch(getConversationSummary(talentEmail)).unwrap();
        } catch (error) {
            dispatch(
                showNotification({
                    message: `获取聊天总结失败: ${error}`,
                    type: 'danger',
                })
            );
        } finally {
            setLoading((prev) => ({ ...prev, summary: false }));
        }
    };

    const handleGetRecommendedReply = async () => {
        if (!chatId || !talentEmail) return;

        setLoading((prev) => ({ ...prev, reply: true }));
        try {
            await dispatch(
                getRecommendedReply({
                    conversation_id: chatId,
                    talent_email: talentEmail,
                })
            ).unwrap();
        } catch (error) {
            dispatch(
                showNotification({
                    message: `获取推荐话术失败: ${error}`,
                    type: 'danger',
                })
            );
        } finally {
            setLoading((prev) => ({ ...prev, reply: false }));
        }
    };

    const handleCopyReply = () => {
        // Get reply text from the appropriate property path
        const replyText = recommendedReply?.reply || '';

        if (replyText) {
            navigator.clipboard
                .writeText(replyText)
                .then(() => {
                    setCopySuccess(true);
                    dispatch(
                        showNotification({
                            message: '已复制到剪贴板',
                            type: 'success',
                        })
                    );
                })
                .catch((err) => {
                    console.error('复制失败:', err);
                    dispatch(
                        showNotification({
                            message: '复制失败，请手动复制',
                            type: 'danger',
                        })
                    );
                });
        }
    };

    return (
        <div className='chat-side-panel d-flex flex-column h-100'>
            {/* 聊天目标 */}
            <div className='panel-section p-3 border-bottom' style={{ flex: '0 0 auto' }}>
                <h5 className='section-title mb-3'>
                    <span className='bg-warning bg-opacity-25 px-2 py-1 rounded'>聊天目标</span>
                </h5>
                <form onSubmit={handleGoalSubmit}>
                    <textarea
                        className='form-control mb-2'
                        rows='3'
                        placeholder='请输入聊天目标...'
                        value={goal}
                        onChange={(e) => setGoal(e.target.value)}
                    ></textarea>
                    <button type='submit' className='btn btn-sm btn-primary' disabled={loading.goal || !goal.trim()}>
                        {loading.goal ? (
                            <>
                                <span
                                    className='spinner-border spinner-border-sm me-2'
                                    role='status'
                                    aria-hidden='true'
                                ></span>
                                提交中...
                            </>
                        ) : (
                            '设置目标'
                        )}
                    </button>
                </form>
                {userGoal && (
                    <div className='mt-2 p-2 bg-light rounded'>
                        <small className='fw-bold'>当前目标：</small>
                        <p className='mb-0 small'>
                            {userGoal.content ||
                                (typeof userGoal === 'string' ? userGoal : userGoal?.data?.content || '')}
                        </p>
                    </div>
                )}
            </div>

            {/* 历史聊天记录总结 */}
            <div className='panel-section p-3 border-bottom' style={{ flex: '1 0 50%', overflowY: 'auto' }}>
                <h5 className='section-title mb-3'>
                    <span className='bg-warning bg-opacity-25 px-2 py-1 rounded'>历史聊天记录总结</span>
                    <button
                        className='btn btn-sm btn-outline-secondary ms-2'
                        onClick={handleGetSummary}
                        disabled={loading.summary || !talentEmail}
                    >
                        {loading.summary ? (
                            <span className='spinner-border spinner-border-sm' role='status' aria-hidden='true'></span>
                        ) : (
                            '刷新'
                        )}
                    </button>
                </h5>
                {loading.summary ? (
                    <div className='text-center py-3'>
                        <div className='spinner-border text-secondary' role='status'>
                            <span className='visually-hidden'>加载中...</span>
                        </div>
                        <p className='mt-2 text-muted'>获取聊天总结...</p>
                    </div>
                ) : conversationSummary?.summary ? (
                    <div className='summary-content'>
                        <SafeMarkdown content={conversationSummary.summary} />
                    </div>
                ) : (
                    <p className='text-muted'>暂无聊天总结</p>
                )}
            </div>

            {/* 推荐聊天话术 */}
            <div className='panel-section p-3' style={{ flex: '1 0 50%', overflowY: 'auto' }}>
                <h5 className='section-title mb-3'>
                    <span className='bg-warning bg-opacity-25 px-2 py-1 rounded'>推荐聊天话术</span>
                    <button
                        className='btn btn-sm btn-outline-secondary ms-2'
                        onClick={handleGetRecommendedReply}
                        disabled={loading.reply || !chatId || !talentEmail}
                    >
                        {loading.reply ? (
                            <span className='spinner-border spinner-border-sm' role='status' aria-hidden='true'></span>
                        ) : (
                            '刷新'
                        )}
                    </button>
                </h5>
                {loading.reply ? (
                    <div className='text-center py-3'>
                        <div className='spinner-border text-secondary' role='status'>
                            <span className='visually-hidden'>加载中...</span>
                        </div>
                        <p className='mt-2 text-muted'>生成话术中...</p>
                    </div>
                ) : recommendedReply?.reply || recommendedReply?.status === 'success' ? (
                    <div
                        className='recommended-reply bg-light p-3 rounded position-relative'
                        style={{ whiteSpace: 'pre-line', fontFamily: 'Arial, sans-serif' }}
                    >
                        <SafeMarkdown content={recommendedReply.reply} />
                        <div className='position-absolute top-0 end-0 m-2'>
                            <button
                                className='btn btn-sm btn-outline-primary'
                                onClick={handleCopyReply}
                                title='复制话术内容'
                            >
                                {copySuccess ? '已复制 ✓' : '复制'}
                            </button>
                        </div>
                    </div>
                ) : (
                    <p className='text-muted'>暂无推荐话术</p>
                )}
            </div>
        </div>
    );
}
