import React, { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';
import { showNotification } from '../../store/notification.slice';
import { fetchAvailableDatasets, fetchChats, createConversation } from '../../store/chat/chat.thunks';
import { setupGmailChat } from '../../store/gmailChat/gmailChat.thunks';
import SvgIcon from '../../components/SvgIcon';

export default function NewChat() {
    const navigate = useNavigate();
    const dispatch = useDispatch();
    const [selectedDatasetIds, setSelectedDatasetIds] = useState([]);
    const [isNavigating, setIsNavigating] = useState(false);

    // Gmail集成相关状态
    const [talentEmail, setTalentEmail] = useState('');
    const [showAuthCodeInput, setShowAuthCodeInput] = useState(false);
    const [authCode, setAuthCode] = useState('');
    const [authUrl, setAuthUrl] = useState('');
    const [isGmailLoading, setIsGmailLoading] = useState(false);

    // 从 Redux store 获取可用知识库数据
    const datasets = useSelector((state) => state.chat.availableDatasets.items || []);
    const isLoading = useSelector((state) => state.chat.availableDatasets.status === 'loading');
    const error = useSelector((state) => state.chat.availableDatasets.error);

    // 获取聊天历史记录
    const chatHistory = useSelector((state) => state.chat.chats.items || []);
    const chatHistoryLoading = useSelector((state) => state.chat.chats.status === 'loading');
    const chatCreationStatus = useSelector((state) => state.chat.messageOperation?.status);

    // Gmail集成状态
    const gmailSetupStatus = useSelector((state) => state.gmailChat?.setup?.status);
    const gmailSetupError = useSelector((state) => state.gmailChat?.setup?.error);

    // 获取可用知识库列表和聊天历史
    useEffect(() => {
        dispatch(fetchAvailableDatasets());
        dispatch(fetchChats({ page: 1, page_size: 50 }));
    }, [dispatch]);

    // 监听错误状态
    useEffect(() => {
        if (error) {
            dispatch(
                showNotification({
                    message: `获取可用知识库列表失败: ${error}`,
                    type: 'danger',
                })
            );
        }
    }, [error, dispatch]);

    // 处理知识库选择（切换选择状态）
    const handleToggleKnowledgeBase = (dataset) => {
        if (isNavigating) return; // 如果正在导航中，阻止操作

        setSelectedDatasetIds((prev) => {
            // 检查是否已经选中
            const isSelected = prev.includes(dataset.id);

            if (isSelected) {
                // 如果已选中，则从数组中移除
                return prev.filter((id) => id !== dataset.id);
            } else {
                // 如果未选中，则添加到数组中
                return [...prev, dataset.id];
            }
        });
    };

    // 处理Gmail集成
    const handleGmailIntegration = async (e) => {
        e.preventDefault();

        if (!talentEmail) {
            dispatch(
                showNotification({
                    message: '请输入达人邮箱',
                    type: 'warning',
                })
            );
            return;
        }

        setIsGmailLoading(true);
        try {
            // 调用Gmail设置API
            const response = await dispatch(setupGmailChat(talentEmail)).unwrap();

            // 如果是没有邮件往来的情况
            if (response.message && response.message.includes('没有找到') && response.knowledge_base_id) {
                dispatch(
                    showNotification({
                        message: response.message,
                        type: 'warning',
                    })
                );

                // 使用获取的知识库ID创建新会话
                try {
                    const conversationResponse = await dispatch(
                        createConversation({
                            dataset_id_list: [response.knowledge_base_id],
                        })
                    ).unwrap();

                    console.log('使用Gmail知识库创建会话成功:', conversationResponse);

                    if (conversationResponse && conversationResponse.conversation_id) {
                        // 保存警告信息到localStorage
                        localStorage.setItem(
                            'gmailNoEmailsWarning',
                            JSON.stringify({
                                message: response.message,
                                troubleshooting: response.troubleshooting || null,
                                chatId: conversationResponse.conversation_id,
                                timestamp: Date.now(),
                                isFirstMessage: true, // 标记这是首次消息
                            })
                        );

                        // 导航到新会话
                        navigate(`/chat/${response.knowledge_base_id}/${conversationResponse.conversation_id}`);
                    } else {
                        throw new Error('未能获取会话ID');
                    }
                } catch (convError) {
                    console.error('创建会话失败:', convError);
                    dispatch(
                        showNotification({
                            message: `创建会话失败: ${convError.message || '请重试'}`,
                            type: 'danger',
                        })
                    );
                }
            }
            // 如果成功返回知识库ID和会话ID
            else if (response && response.data && response.data.knowledge_base_id && response.data.conversation_id) {
                dispatch(
                    showNotification({
                        message: response.message || 'Gmail集成成功',
                        type: 'success',
                    })
                );
                // 导航到新会话
                navigate(`/chat/${response.data.knowledge_base_id}/${response.data.conversation_id}`);
            } else if (response && response.status === 'authorization_required' && response.auth_url) {
                // 需要授权
                setAuthUrl(response.auth_url);
                setShowAuthCodeInput(true);
                // 打开授权URL
                window.open(response.auth_url, '_blank');
                dispatch(
                    showNotification({
                        message: '需要Gmail授权，请在新打开的窗口中完成授权并获取授权码',
                        type: 'info',
                    })
                );
            }
        } catch (error) {
            console.error('Gmail集成失败:', error);
            dispatch(
                showNotification({
                    message: `Gmail集成失败: ${error.message || '请重试'}`,
                    type: 'danger',
                })
            );
        } finally {
            setIsGmailLoading(false);
        }
    };

    // 处理授权码提交
    const handleAuthCodeSubmit = async (e) => {
        e.preventDefault();

        if (!authCode) {
            dispatch(
                showNotification({
                    message: '请输入授权码',
                    type: 'warning',
                })
            );
            return;
        }

        setIsGmailLoading(true);
        try {
            // 使用授权码再次调用Gmail设置API
            const response = await dispatch(
                setupGmailChat({
                    talent_gmail: talentEmail,
                    auth_code: authCode,
                })
            ).unwrap();

            if (response && response.data && response.data.knowledge_base_id && response.data.conversation_id) {
                dispatch(
                    showNotification({
                        message: response.message || 'Gmail集成成功',
                        type: 'success',
                    })
                );
                // 导航到新会话
                navigate(`/chat/${response.data.knowledge_base_id}/${response.data.conversation_id}`);
            } else {
                throw new Error('未能获取会话信息');
            }
        } catch (error) {
            console.error('授权码提交失败:', error);
            dispatch(
                showNotification({
                    message: `授权码提交失败: ${error.message || '请重试'}`,
                    type: 'danger',
                })
            );
        } finally {
            setIsGmailLoading(false);
        }
    };

    // 开始聊天
    const handleStartChat = async () => {
        if (selectedDatasetIds.length === 0) {
            dispatch(
                showNotification({
                    message: '请至少选择一个知识库',
                    type: 'warning',
                })
            );
            return;
        }

        if (isNavigating) return; // 防止重复点击

        try {
            setIsNavigating(true);

            // 打印调试信息
            console.log('选中的知识库ID:', selectedDatasetIds);

            // 检查是否已存在包含所有选中知识库的聊天记录
            // 注意：这里的逻辑简化了，实际可能需要更复杂的匹配算法
            const existingChat = chatHistory.find((chat) => {
                // 检查聊天记录中的知识库是否完全匹配当前选择
                if (chat.datasets && Array.isArray(chat.datasets)) {
                    const chatDatasetIds = chat.datasets.map((ds) => ds.id);
                    return (
                        chatDatasetIds.length === selectedDatasetIds.length &&
                        selectedDatasetIds.every((id) => chatDatasetIds.includes(id))
                    );
                }

                // 兼容旧格式
                if (chat.dataset_id_list && Array.isArray(chat.dataset_id_list)) {
                    const formattedSelectedIds = selectedDatasetIds.map((id) => id.replace(/-/g, ''));
                    return (
                        chat.dataset_id_list.length === formattedSelectedIds.length &&
                        formattedSelectedIds.every((id) => chat.dataset_id_list.includes(id))
                    );
                }

                return false;
            });

            if (existingChat) {
                // 找到现有聊天记录，导航到该聊天页面
                // 使用所有知识库ID作为URL参数，以逗号分隔
                const knowledgeBaseIdsParam = selectedDatasetIds.join(',');
                console.log(
                    `找到现有聊天记录，直接导航到 /chat/${knowledgeBaseIdsParam}/${existingChat.conversation_id}`
                );
                navigate(`/chat/${knowledgeBaseIdsParam}/${existingChat.conversation_id}`);
            } else {
                // 没有找到现有聊天记录，创建新的聊天
                console.log(`未找到现有聊天记录，创建新会话，选中的知识库ID: ${selectedDatasetIds.join(', ')}`);

                try {
                    // 调用createConversation创建新会话（不发送消息）
                    const response = await dispatch(
                        createConversation({
                            dataset_id_list: selectedDatasetIds,
                        })
                    ).unwrap();

                    console.log('创建会话响应:', response);

                    if (response && response.conversation_id) {
                        // 使用所有知识库ID作为URL参数，以逗号分隔
                        const knowledgeBaseIdsParam = selectedDatasetIds.join(',');
                        console.log(`创建会话成功，导航到 /chat/${knowledgeBaseIdsParam}/${response.conversation_id}`);
                        navigate(`/chat/${knowledgeBaseIdsParam}/${response.conversation_id}`);
                    } else {
                        throw new Error('未能获取会话ID：' + JSON.stringify(response));
                    }
                } catch (apiError) {
                    // 专门处理API调用错误
                    console.error('API调用失败:', apiError);
                    throw new Error(`API调用失败: ${apiError.message || '未知错误'}`);
                }
            }
        } catch (error) {
            console.error('导航或创建聊天失败:', error);

            // 添加更详细的错误日志
            if (error.stack) {
                console.error('错误堆栈:', error.stack);
            }

            // 显示更友好的错误消息
            dispatch(
                showNotification({
                    message: `创建聊天失败: ${error.message || '请重试'}`,
                    type: 'danger',
                })
            );
        } finally {
            setIsNavigating(false);
        }
    };

    // 渲染加载状态
    if (isLoading || chatHistoryLoading) {
        return (
            <div className='container-fluid px-4 py-5 text-center'>
                <div className='spinner-border' role='status'>
                    <span className='visually-hidden'>加载中...</span>
                </div>
            </div>
        );
    }

    return (
        <div className='container-fluid px-4 py-5'>
            {/* 导航中的遮罩层 */}
            {isNavigating && (
                <div
                    className='position-fixed top-0 start-0 w-100 h-100 d-flex justify-content-center align-items-center'
                    style={{
                        backgroundColor: 'rgba(255, 255, 255, 0.7)',
                        zIndex: 1050,
                    }}
                >
                    <div className='text-center'>
                        <div className='spinner-border mb-2' role='status'>
                            <span className='visually-hidden'>加载中...</span>
                        </div>
                        <div>正在加载聊天界面...</div>
                    </div>
                </div>
            )}

            {/* Gmail集成模块 */}
            <div className='card mb-5 shadow-sm'>
                <div className='card-body'>
                    <h4 className='card-title mb-4'>Gmail邮件集成</h4>

                    {!showAuthCodeInput ? (
                        <form onSubmit={handleGmailIntegration} className='mb-3'>
                            <div className='mb-3'>
                                <label htmlFor='talentEmail' className='form-label'>
                                    达人邮箱地址
                                </label>
                                <div className='input-group'>
                                    <input
                                        type='email'
                                        className='form-control'
                                        id='talentEmail'
                                        placeholder='请输入正在联络的达人邮箱'
                                        value={talentEmail}
                                        onChange={(e) => setTalentEmail(e.target.value)}
                                        disabled={isGmailLoading}
                                        required
                                    />
                                    <button
                                        className='btn btn-outline-dark'
                                        type='submit'
                                        disabled={isGmailLoading || !talentEmail}
                                    >
                                        {isGmailLoading ? (
                                            <>
                                                <span
                                                    className='spinner-border spinner-border-sm me-2'
                                                    role='status'
                                                    aria-hidden='true'
                                                ></span>
                                                处理中...
                                            </>
                                        ) : (
                                            '创建邮件会话'
                                        )}
                                    </button>
                                </div>
                                <div className='form-text'>将创建一个包含您与该达人Gmail邮件沟通记录的知识库及会话</div>
                            </div>
                        </form>
                    ) : (
                        <form onSubmit={handleAuthCodeSubmit} className='mb-3'>
                            <div className='alert alert-info mb-3'>
                                <p className='mb-2'>
                                    <strong>需要Gmail授权</strong>
                                </p>
                                <p className='mb-2'>系统已在新窗口打开Gmail授权页面，请完成授权并获取授权码。</p>
                                <p className='mb-0'>
                                    <a
                                        href={authUrl}
                                        target='_blank'
                                        rel='noopener noreferrer'
                                        className='btn btn-sm btn-outline-primary'
                                    >
                                        重新打开授权页面
                                    </a>
                                </p>
                            </div>

                            <div className='mb-3'>
                                <label htmlFor='authCode' className='form-label'>
                                    授权码
                                </label>
                                <div className='input-group'>
                                    <input
                                        type='text'
                                        className='form-control'
                                        id='authCode'
                                        placeholder='请输入Gmail授权页面提供的授权码'
                                        value={authCode}
                                        onChange={(e) => setAuthCode(e.target.value)}
                                        disabled={isGmailLoading}
                                        required
                                    />
                                    <button
                                        className='btn btn-outline-success'
                                        type='submit'
                                        disabled={isGmailLoading || !authCode}
                                    >
                                        {isGmailLoading ? (
                                            <>
                                                <span
                                                    className='spinner-border spinner-border-sm me-2'
                                                    role='status'
                                                    aria-hidden='true'
                                                ></span>
                                                处理中...
                                            </>
                                        ) : (
                                            '提交授权码'
                                        )}
                                    </button>
                                </div>
                            </div>

                            <button
                                type='button'
                                className='btn btn-link ps-0'
                                onClick={() => {
                                    setShowAuthCodeInput(false);
                                    setAuthCode('');
                                    setAuthUrl('');
                                }}
                            >
                                &laquo; 返回输入达人邮箱
                            </button>
                        </form>
                    )}
                </div>
            </div>

            <div className='d-flex justify-content-between align-items-center mb-4'>
                <h4 className='m-0'>选择知识库开始聊天</h4>
                <button
                    className='btn btn-dark'
                    onClick={handleStartChat}
                    disabled={selectedDatasetIds.length === 0 || isNavigating}
                >
                    {isNavigating ? (
                        <>
                            <span
                                className='spinner-border spinner-border-sm me-2'
                                role='status'
                                aria-hidden='true'
                            ></span>
                            加载中...
                        </>
                    ) : (
                        <>开始聊天 {selectedDatasetIds.length > 0 && `(${selectedDatasetIds.length})`}</>
                    )}
                </button>
            </div>

            {selectedDatasetIds.length > 0 && (
                <div className='alert alert-dark mb-4'>已选择 {selectedDatasetIds.length} 个知识库</div>
            )}

            <div className='row row-cols-1 row-cols-md-2 row-cols-lg-3 g-4 justify-content-center'>
                {datasets.length > 0 ? (
                    datasets.map((dataset) => {
                        const isSelected = selectedDatasetIds.includes(dataset.id);
                        return (
                            <div key={dataset.id} className='col'>
                                <div
                                    className={`card h-100 shadow-sm ${!isNavigating ? 'cursor-pointer' : ''} ${
                                        isSelected ? 'border-gray border-2' : 'border-0'
                                    }`}
                                    onClick={() => handleToggleKnowledgeBase(dataset)}
                                    style={{ opacity: isNavigating && !isSelected ? 0.6 : 1 }}
                                >
                                    <div className='card-body'>
                                        <h5 className='card-title d-flex justify-content-between align-items-center'>
                                            {dataset.name}
                                            {isSelected && <SvgIcon className='check-circle text-primary' />}
                                        </h5>
                                        <p className='card-text text-muted'>
                                            {dataset.desc || dataset.description || ''}
                                        </p>
                                        <div className='text-muted small d-flex align-items-center gap-2'>
                                            <span className='d-flex align-items-center gap-1'>
                                                {dataset.department || ''}
                                            </span>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        );
                    })
                ) : (
                    <div className='col-12'>
                        <div className='alert alert-warning'>暂无可访问的知识库，请先申请知识库访问权限</div>
                    </div>
                )}
            </div>
        </div>
    );
}
