import axios from 'axios';
import CryptoJS from 'crypto-js';

const secretKey = import.meta.env.VITE_SECRETKEY;

// API连接状态
let isServerDown = false;
let hasCheckedServer = false;

// Create Axios instance with base URL
const api = axios.create({
    baseURL: '/api',
    withCredentials: true, // Include cookies if needed
});

// Request Interceptor
api.interceptors.request.use(
    (config) => {
        const encryptedToken = sessionStorage.getItem('token') || '';
        if (encryptedToken) {
            const decryptedToken = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
            config.headers.Authorization = `Token ${decryptedToken}`;
        }
        return config;
    },
    (error) => {
        console.error('Request error:', error);
        return Promise.reject(error);
    }
);

// Response Interceptor
api.interceptors.response.use(
    (response) => {
        // 如果成功收到响应，表示服务器正常工作
        if (!hasCheckedServer) {
            console.log('Server is up and running');
            isServerDown = false;
            hasCheckedServer = true;
        }
        return response;
    },
    (error) => {
        // Handle errors in the response
        if (error.response) {
            // monitor /verify
            if (error.response.status === 401 && error.config.url === '/auth/verify-token/') {
                if (window.location.pathname !== '/login' && window.location.pathname !== '/signup') {
                    window.location.href = '/login';
                }
            }

            // The request was made and the server responded with a status code
            console.error('API Error Response:', error.response.status, error.response.data.message);
            // alert(`Error: ${error.response.status} - ${error.response.data.message || 'Something went wrong'}`);
        } else if (error.request) {
            // The request was made but no response was received
            console.error('API Error Request:', error.request);
            // alert('Network error: No response from server');
        } else {
            // Something happened in setting up the request
            console.error('API Error Message:', error.message);
            // alert('Error: ' + error.message);
        }
        return Promise.reject(error); // Reject the promise
    }
);


// Define common HTTP methods 
const get = async (url, params = {}) => {
    try {
        const res = await api.get(url, { ...params });
        return res.data;
    } catch (error) {
        throw error;
    }
};

// Handle POST requests for JSON data 
const post = async (url, data, isMultipart = false) => {
    try {
        const headers = isMultipart
            ? { 'Content-Type': 'multipart/form-data' } // For file uploads
            : { 'Content-Type': 'application/json' }; // For JSON data

        const res = await api.post(url, data, { headers });
        return res.data;
    } catch (error) {
        throw error;
    }
};

// Handle PUT requests
const put = async (url, data) => {
    try {
        const res = await api.put(url, data, {
            headers: { 'Content-Type': 'application/json' },
        });
        return res.data;
    } catch (error) {
        throw error;
    }
};

// Handle DELETE requests
const del = async (url) => {
    try {
        const res = await api.delete(url);
        return res.data;
    } catch (error) {
        throw error;
    }
};

const upload = async (url, data) => {
    try {
        const axiosInstance = await axios.create({
            baseURL: '/api',
            headers: {
                'Content-Type': 'multipart/form-data',
            },
        });
        const res = await axiosInstance.post(url, data);
        return res.data;
    } catch (error) {
        throw error;
    }
};
// Handle streaming requests
const streamRequest = async (url, data, onChunk, onError) => {
    try {

        // 获取认证Token
        const encryptedToken = sessionStorage.getItem('token') || '';
        let token = '';
        if (encryptedToken) {
            token = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
        }

        // 使用fetch API进行流式请求
        const response = await fetch(`/api${url}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': token ? `Token ${token}` : '',
            },
            body: JSON.stringify(data),
        });

        if (!response.ok) {
            throw new Error(`HTTP error! Status: ${response.status}`);
        }

        // 获取响应体的reader
        const reader = response.body.getReader();
        const decoder = new TextDecoder();
        let buffer = '';
        let conversationId = null;

        // 处理流式数据
        while (true) {
            const { done, value } = await reader.read();
            if (done) break;

            // 解码并处理数据
            const chunk = decoder.decode(value, { stream: true });
            buffer += chunk;

            // 按行分割并处理JSON
            const lines = buffer.split('\n');
            buffer = lines.pop() || ''; // 保留最后一行（可能不完整）

            for (const line of lines) {
                if (!line.trim()) continue;
                
                try {
                    // 检查是否为SSE格式(data: {...})
                    let jsonStr = line;
                    if (line.startsWith('data:')) {
                        // 提取data:后面的JSON部分
                        jsonStr = line.substring(5).trim();
                        console.log('检测到SSE格式数据，提取JSON:', jsonStr);
                    }
                    
                    // 尝试解析JSON
                    const data = JSON.parse(jsonStr);
                    if (data.code === 200 && data.data && data.data.conversation_id) {
                        conversationId = data.data.conversation_id;
                    }
                    onChunk(jsonStr);
                } catch (e) {
                    console.warn('Failed to parse JSON:', line, e);
                }
            }
        }

        return { success: true, conversation_id: conversationId };
    } catch (error) {
        console.error('Streaming request failed:', error);
        if (onError) {
            onError(error);
        }
        throw error;
    }
};

// 权限相关API
export const applyPermission = (data) => {
    return post('/permissions/', data);
};

export const updatePermission = (data) => {
    return post('/permissions/update_permission/', data);
};

export const approvePermission = (permissionId) => {
    return post(`/permissions/approve_permission/${permissionId}`);
};

export const rejectPermission = (permissionId) => {
    return post(`/permissions/reject_permission/${permissionId}`);
};

export { get, post, put, del, upload, streamRequest };
