import { createSlice } from '@reduxjs/toolkit';
import {
    fetchAvailableDatasets,
    fetchChats,
    updateChat,
    deleteChat,
    createChatRecord,
    fetchConversationDetail,
    createConversation,
} from './chat.thunks';
import { fetchMessages, sendMessage } from './chat.messages.thunks';

// 初始状态
const initialState = {
    // 聊天列表，包含所有聊天及其消息
    chats: {
        items: [], // 每个chat对象包含conversation_id, datasets, create_time, messages等
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
        pagination: {
            total: 0,
            page: 1,
            page_size: 10,
        },
    },
    // 当前活跃聊天的ID
    activeConversationId: null,
    // 消息发送状态
    messageOperation: {
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
        streamingMessageId: null, // 当前正在流式传输的消息ID（如果有）
    },
    // 可用于聊天的知识库列表
    availableDatasets: {
        items: [],
        status: 'idle',
        error: null,
    },
    // 聊天操作状态（创建、更新、删除）
    chatOperation: {
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
    },
};

// 辅助函数：查找聊天索引
const findChatIndex = (state, conversationId) => {
    return state.chats.items.findIndex((chat) => chat.conversation_id === conversationId);
};

// 创建 slice
const chatSlice = createSlice({
    name: 'chat',
    initialState,
    reducers: {
        // 重置操作状态
        resetChatOperation: (state) => {
            state.chatOperation.status = 'idle';
            state.chatOperation.error = null;
        },

        // 设置当前活跃聊天
        setActiveChat: (state, action) => {
            state.activeConversationId = action.payload;
        },

        // 重置消息操作状态
        resetMessageOperation: (state) => {
            state.messageOperation.status = 'idle';
            state.messageOperation.error = null;
            state.messageOperation.streamingMessageId = null;
        },

        // 添加消息到特定聊天
        addMessage: (state, action) => {
            const { conversationId, message } = action.payload;
            const chatIndex = findChatIndex(state, conversationId);

            if (chatIndex !== -1) {
                // 确保chat有messages数组
                if (!state.chats.items[chatIndex].messages) {
                    state.chats.items[chatIndex].messages = [];
                }

                // 添加消息
                state.chats.items[chatIndex].messages.push(message);

                // 更新最后一条消息和消息计数
                state.chats.items[chatIndex].last_message = message.content;
                state.chats.items[chatIndex].message_count = (state.chats.items[chatIndex].message_count || 0) + 1;

                // 如果是助手消息且正在流式传输，记录ID
                if (message.role === 'assistant' && message.is_streaming) {
                    state.messageOperation.streamingMessageId = message.id;
                    state.messageOperation.status = 'loading';
                }
            }
        },

        // 更新消息（用于流式传输）
        updateMessage: (state, action) => {
            const { conversationId, messageId, updates, serverMessageId } = action.payload;
            const chatIndex = findChatIndex(state, conversationId);

            if (chatIndex !== -1 && state.chats.items[chatIndex].messages) {
                // 首先尝试使用服务器返回的ID找到消息
                let messageIndex = -1;

                if (serverMessageId) {
                    // 如果提供了服务器ID，优先使用它查找消息
                    messageIndex = state.chats.items[chatIndex].messages.findIndex(
                        (msg) => msg.id === serverMessageId || msg.server_id === serverMessageId
                    );
                }

                // 如果没找到或没提供服务器ID，则使用客户端生成的ID
                if (messageIndex === -1) {
                    messageIndex = state.chats.items[chatIndex].messages.findIndex((msg) => msg.id === messageId);
                }

                if (messageIndex !== -1) {
                    // 更新现有消息
                    const updatedMessage = {
                        ...state.chats.items[chatIndex].messages[messageIndex],
                        ...updates,
                    };

                    // 如果收到了服务器ID且消息没有server_id字段，添加它
                    if (serverMessageId && !updatedMessage.server_id) {
                        updatedMessage.server_id = serverMessageId;
                    }

                    state.chats.items[chatIndex].messages[messageIndex] = updatedMessage;

                    // 如果流式传输结束，更新状态
                    if (
                        updates.is_streaming === false &&
                        (messageId === state.messageOperation.streamingMessageId ||
                            serverMessageId === state.messageOperation.streamingMessageId)
                    ) {
                        state.messageOperation.status = 'succeeded';
                        state.messageOperation.streamingMessageId = null;
                    }

                    // 如果更新了内容，更新最后一条消息
                    if (updates.content) {
                        state.chats.items[chatIndex].last_message = updates.content;
                    }
                }
            }
        },
    },
    extraReducers: (builder) => {
        // 获取聊天列表
        builder
            .addCase(fetchChats.pending, (state) => {
                state.chats.status = 'loading';
            })
            .addCase(fetchChats.fulfilled, (state, action) => {
                state.chats.status = 'succeeded';

                // 检查是否是追加模式
                if (action.payload.append) {
                    // 追加模式：将新结果添加到现有列表的前面
                    state.chats.items = [...action.payload.results, ...state.chats.items];
                } else {
                    // 替换模式：使用新结果替换整个列表
                    state.chats.items = action.payload.results;
                    state.chats.pagination.total = action.payload.total;
                    state.chats.pagination.page = action.payload.page;
                    state.chats.pagination.page_size = action.payload.page_size;
                }
            })
            .addCase(fetchChats.rejected, (state, action) => {
                state.chats.status = 'failed';
                state.chats.error = action.payload || action.error.message;
            })

            // 删除聊天
            .addCase(deleteChat.pending, (state) => {
                state.chatOperation.status = 'loading';
            })
            .addCase(deleteChat.fulfilled, (state, action) => {
                state.chatOperation.status = 'succeeded';
                // 删除聊天
                state.chats.items = state.chats.items.filter((chat) => chat.conversation_id !== action.payload);

                if (state.chats.pagination.total > 0) {
                    state.chats.pagination.total -= 1;
                }

                // 如果删除的是当前活跃聊天，重置activeConversationId
                if (state.activeConversationId === action.payload) {
                    state.activeConversationId = null;
                }
            })
            .addCase(deleteChat.rejected, (state, action) => {
                state.chatOperation.status = 'failed';
                state.chatOperation.error = action.payload || action.error.message;
            })

            // 更新聊天
            .addCase(updateChat.pending, (state) => {
                state.chatOperation.status = 'loading';
            })
            .addCase(updateChat.fulfilled, (state, action) => {
                state.chatOperation.status = 'succeeded';
                const chatIndex = findChatIndex(state, action.payload.conversation_id);
                if (chatIndex !== -1) {
                    // 保留messages字段，避免覆盖
                    const existingMessages = state.chats.items[chatIndex].messages;
                    state.chats.items[chatIndex] = {
                        ...action.payload,
                        messages: existingMessages || [],
                    };
                }
            })
            .addCase(updateChat.rejected, (state, action) => {
                state.chatOperation.status = 'failed';
                state.chatOperation.error = action.payload || action.error.message;
            })

            // 获取聊天消息
            .addCase(fetchMessages.pending, (state) => {
                state.messageOperation.status = 'loading';
            })
            .addCase(fetchMessages.fulfilled, (state, action) => {
                state.messageOperation.status = 'succeeded';
                // 假设action.meta.arg是conversationId
                const conversationId = action.meta.arg;
                const chatIndex = findChatIndex(state, conversationId);

                if (chatIndex !== -1) {
                    state.chats.items[chatIndex].messages = action.payload;
                }
            })
            .addCase(fetchMessages.rejected, (state, action) => {
                state.messageOperation.status = 'failed';
                state.messageOperation.error = action.error.message;
            })

            // 发送聊天消息
            .addCase(sendMessage.pending, (state) => {
                state.messageOperation.status = 'loading';
            })
            .addCase(sendMessage.fulfilled, (state, action) => {
                state.messageOperation.status = 'succeeded';
                // 假设action.meta.arg包含chatId
                const { chatId } = action.meta.arg;
                const chatIndex = findChatIndex(state, chatId);

                if (chatIndex !== -1) {
                    // 确保chat有messages数组
                    if (!state.chats.items[chatIndex].messages) {
                        state.chats.items[chatIndex].messages = [];
                    }

                    // 检查消息是否已存在
                    const messageExists = state.chats.items[chatIndex].messages.some(
                        (msg) => msg.content === action.payload.content && msg.role === action.payload.role
                    );

                    if (!messageExists) {
                        state.chats.items[chatIndex].messages.push(action.payload);
                        state.chats.items[chatIndex].last_message = action.payload.content;
                        state.chats.items[chatIndex].message_count =
                            (state.chats.items[chatIndex].message_count || 0) + 1;
                    }
                }
            })
            .addCase(sendMessage.rejected, (state, action) => {
                state.messageOperation.status = 'failed';
                state.messageOperation.error = action.error.message;
            })

            // 处理创建聊天记录
            .addCase(createChatRecord.pending, (state) => {
                state.messageOperation.status = 'loading';
            })
            .addCase(createChatRecord.fulfilled, (state, action) => {
                // 聊天创建成功，但消息状态由addMessage和updateMessage处理
                state.activeConversationId = action.payload.conversation_id;
            })
            .addCase(createChatRecord.rejected, (state, action) => {
                state.messageOperation.status = 'failed';
                state.messageOperation.error = action.error.message;
            })

            // 处理创建会话
            .addCase(createConversation.pending, (state) => {
                state.chatOperation.status = 'loading';
            })
            .addCase(createConversation.fulfilled, (state, action) => {
                state.chatOperation.status = 'succeeded';
                state.activeConversationId = action.payload.conversation_id;

                // 在执行createConversation时，已经通过dispatch添加了新聊天到列表
                // 所以这里只需确保当前激活的聊天ID已设置
            })
            .addCase(createConversation.rejected, (state, action) => {
                state.chatOperation.status = 'failed';
                state.chatOperation.error = action.payload || action.error.message;
            })

            // 处理获取可用知识库
            .addCase(fetchAvailableDatasets.pending, (state) => {
                state.availableDatasets.status = 'loading';
            })
            .addCase(fetchAvailableDatasets.fulfilled, (state, action) => {
                state.availableDatasets.status = 'succeeded';
                state.availableDatasets.items = action.payload || [];
            })
            .addCase(fetchAvailableDatasets.rejected, (state, action) => {
                state.availableDatasets.status = 'failed';
                state.availableDatasets.error = action.payload || '获取可用知识库失败';
            })

            // 获取会话详情
            .addCase(fetchConversationDetail.pending, (state) => {
                // 设置加载状态
            })
            .addCase(fetchConversationDetail.fulfilled, (state, action) => {
                // 如果有返回数据
                if (action.payload) {
                    const conversationId = action.payload.conversation_id;
                    const chatIndex = findChatIndex(state, conversationId);

                    if (chatIndex !== -1) {
                        // 更新现有聊天
                        state.chats.items[chatIndex] = {
                            ...state.chats.items[chatIndex],
                            ...action.payload,
                        };
                    } else {
                        // 添加新聊天
                        state.chats.items.push(action.payload);
                    }

                    // 设置为当前活跃聊天
                    state.activeConversationId = conversationId;
                }
            })
            .addCase(fetchConversationDetail.rejected, (state, action) => {
                // 仅在操作失败时设置错误状态
            });
    },
});

// 导出 actions
export const { resetChatOperation, setActiveChat, resetMessageOperation, addMessage, updateMessage } =
    chatSlice.actions;

// 导出 reducer
export default chatSlice.reducer;
